﻿using System;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Threading;

using mshtml;

namespace cleverbridge.Games.TicTacToe
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        private readonly DispatcherTimer _dispatcherTimer;
        private int _frameHeight = 605;
        public int FrameHeight
        {
            get => _frameHeight;
            set
            {
                if (_frameHeight != value)
                {
                    if (value >= 100 && value <= 2000)
                        _frameHeight = value;
                    OnPropertyChanged();
                    OnPropertyChanged(nameof(WindowHeight));
                }
            }
        }

        private int _frameWidth = 815;
        public int FrameWidth
        {
            get => _frameWidth;
            set
            {
                if (_frameWidth != value)
                {
                    if (value >= 100 && value <= 2000)
                        _frameWidth = value;
                    OnPropertyChanged();
                    OnPropertyChanged(nameof(WindowWidth));
                }
            }
        }

        public int WindowWidth => FrameWidth + 60;

        public int WindowHeight => FrameHeight + 95;

        private Uri _link = new Uri("https://www.cleverbridge.com/864/purl-tictactoe");
        public Uri Link
        {
            get => _link;
            set
            {
                if (_link != value)
                {
                    _link = value;
                    OnPropertyChanged();
                }
            }
        }

        private Visibility _settingsVisibility = Visibility.Collapsed;
        public Visibility SettingsVisibility
        {
            get => _settingsVisibility;
            set
            {
                if (_settingsVisibility != value)
                {
                    _settingsVisibility = value;
                    OnPropertyChanged();
                }
            }
        }

        public MainWindow()
        {
            SetBrowserVersion(GetArgument("-v").ToLower());
            InitializeComponent();

            SetBrowserComboBox();

            SetDefaultFontFamily();

            _dispatcherTimer = new DispatcherTimer
            {
                Interval = TimeSpan.FromMilliseconds(300)
            };
            _dispatcherTimer.Tick += new EventHandler(_dispatcherTimer_Tick);

            try
            {
                if (File.Exists(LicenseKeyPath))
                    _buyLabel.Visibility = Visibility.Collapsed;
            }
            catch
            {
            }

            if (int.TryParse(GetArgument("-h"), out int height))
            {
                FrameHeight = height;
            }

            if (int.TryParse(GetArgument("-w"), out int width))
            {
                FrameWidth = width;
            }

            var argument = GetArgument("-u");
            if (string.IsNullOrWhiteSpace(argument))
                argument = "https://www.cleverbridge.com/864/purl-tictactoe";
            Link = new Uri(argument);

            if (Array.IndexOf(Environment.GetCommandLineArgs(), "-cb") >= 0)
            {
                SettingsVisibility = Visibility.Visible;
            }
        }

        private void _buyButton_Click(
            object sender,
            RoutedEventArgs e)
        {
            if (WindowWidth + 20 > Width)
                Width = WindowWidth + 20;
            if (WindowHeight + 40 > Height)
                Height = WindowHeight + 40;

            _frame.Source = Link;

            _dispatcherTimer.Start();

            _orderProcessLabel.Visibility = Visibility.Visible;

            var fadeInAnimation = new DoubleAnimation
            {
                To = 1,
                Duration = new Duration(TimeSpan.FromSeconds(0.3))
            };

            _orderProcessLabel.BeginAnimation(OpacityProperty, fadeInAnimation);
            _browserComboBox.IsEnabled = false;
        }

        private void _closeOrderProcessButton_Click(
            object sender,
            RoutedEventArgs e)
        {
            _dispatcherTimer.Stop();

            var fadeOutAnimation = new DoubleAnimation
            {
                To = 0,
                Duration = new Duration(TimeSpan.FromSeconds(0.2))
            };
            fadeOutAnimation.Completed += new EventHandler(fadeOutAnimationCompleted);

            _orderProcessLabel.BeginAnimation(OpacityProperty, fadeOutAnimation);
        }

        private void _dispatcherTimer_Tick(
            object sender,
            EventArgs e)
        {
            const string LicenseKeyStart = "<!-- LICENSEKEY=";
            const string LicenseKeyEnd = " -->";

            if (_frame.Content is WebBrowser)
            {
                var webBrowser = (WebBrowser)_frame.Content;
                if (webBrowser.Document is HTMLDocument)
                {
                    var htmlDocument = (HTMLDocument)webBrowser.Document;
                    if (htmlDocument.documentElement != null)
                    {
                        var innerHtml = htmlDocument.documentElement.innerHTML;

                        var startIndex = innerHtml.IndexOf(LicenseKeyStart);
                        if (startIndex >= 0)
                        {
                            var endIndex = innerHtml.IndexOf(LicenseKeyEnd, startIndex + LicenseKeyStart.Length);
                            if (endIndex >= 0)
                            {
                                var licenseKey = innerHtml.Substring(startIndex + LicenseKeyStart.Length, endIndex - (startIndex + LicenseKeyStart.Length));

                                _dispatcherTimer.Stop();

                                try
                                {
                                    File.WriteAllText(LicenseKeyPath, licenseKey);
                                    _buyLabel.Visibility = Visibility.Collapsed;
                                }
                                catch
                                {
                                }

                                var licenseeKeyWindow = new LicenseeKeyWindow
                                {
                                    LicenseeKey = licenseKey,
                                    Owner = this
                                };

                                licenseeKeyWindow.ShowDialog();
                            }
                        }
                    }
                }
            }
        }

        void fadeOutAnimationCompleted(
            object sender,
            EventArgs e)
        {
            _orderProcessLabel.Visibility = Visibility.Collapsed;
        }

        private string GetArgument(
            string option)
        {
            var args = Environment.GetCommandLineArgs();
            if (!option.StartsWith("-"))
                option = "-" + option;
            int index = Array.IndexOf(args, option);
            if (index >= 0 && args.Length > index + 1)
                return args[index + 1];
            return string.Empty;
        }

        private void SetDefaultFontFamily()
        {
            TextElement.FontFamilyProperty.OverrideMetadata(
                typeof(TextElement),
                new FrameworkPropertyMetadata(
                new FontFamily("Arial")));

            TextBlock.FontFamilyProperty.OverrideMetadata(
                typeof(TextBlock),
                new FrameworkPropertyMetadata(
                new FontFamily("Arial")));

            Label.FontFamilyProperty.OverrideMetadata(
                typeof(Label),
                new FrameworkPropertyMetadata(
                new FontFamily("Arial")));
        }

        public static string LicenseKeyPath
        {
            get
            {
                var stringBuilder = new StringBuilder();

                stringBuilder.Append(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location));
                stringBuilder.Append(Path.DirectorySeparatorChar);
                stringBuilder.Append("License.txt");

                return stringBuilder.ToString();
            }
        }

        private void BrowserComboBox_OnSelectionChanged(
            object sender,
            SelectionChangedEventArgs e)
        {
            var item = _browserComboBox.SelectedItem as ComboBoxItem;
            if (item == null)
                return;

            SetBrowserVersion(item.Content.ToString());
        }

        private static void SetBrowserVersion(string version)
        {
            switch (version)
            {
                case "edge":
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(12);
                    break;
                case "ie11":
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(11);
                    break;
                case "ie10":
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(10);
                    break;
                case "ie9":
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(9);
                    break;
                case "ie8":
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(8);
                    break;
                case "ie7":
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(7);
                    break;
                default:
                    InternetExplorerBrowserEmulation.SetBrowserEmulationVersion(11);
                    break;
            }
        }

        private void SetBrowserComboBox()
        {
            _browserComboBox.SelectionChanged -= BrowserComboBox_OnSelectionChanged;
            switch (GetArgument("-v").ToLower())
            {
                case "ie11":
                    _browserComboBox.SelectedIndex = 4;
                    break;
                case "ie10":
                    _browserComboBox.SelectedIndex = 3;
                    break;
                case "ie9":
                    _browserComboBox.SelectedIndex = 2;
                    break;
                case "ie8":
                    _browserComboBox.SelectedIndex = 1;
                    break;
                case "ie7":
                    _browserComboBox.SelectedIndex = 0;
                    break;
                default:
                    _browserComboBox.SelectedIndex = 4;
                    break;
            }
            _browserComboBox.SelectionChanged += BrowserComboBox_OnSelectionChanged;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(
            [CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

    }
}